@@ -1,7 +1,6 @@
 # encoding: utf-8
 from mo_future import text
 
-from mo_parsing.cache import packrat_cache
 from mo_parsing.core import ParserElement
 from mo_parsing.exceptions import ParseException
 from mo_parsing.utils import (
@@ -26,37 +25,24 @@ class Debugger(object):
 
 
 def _debug_parse(self, string, start, doActions=True):
-    lookup = (self, string, start, doActions)
-    value = packrat_cache.get(lookup)
-    if value is not None:
-        if isinstance(value, Exception):
-            raise value
-        return value
-
+    _try(self, start, string)
+    loc = self.engine.skip(string, start)
     try:
-        _try(self, start, string)
-        loc = self.engine.skip(string, start)
-        try:
-            tokens = self.parseImpl(string, loc, doActions)
-        except Exception as cause:
-            loc = self.engine.skip(string, start)
-            self.parser_config.failAction and self.parser_config.failAction(self, start, string, cause)
-            fail(self, start, string, cause)
-            raise
-
-        if self.parseAction and (doActions or self.parser_config.callDuringTry):
-            try:
-                for fn in self.parseAction:
-                    tokens = fn(tokens, start, string)
-            except Exception as cause:
-                fail(self, start, string, cause)
-                raise
-        match(self, loc, tokens.end, string, tokens)
-    except ParseException as cause:
-        packrat_cache.set(lookup, cause)
+        tokens = self.parseImpl(string, loc, doActions)
+    except Exception as cause:
+        self.parser_config.failAction and self.parser_config.failAction(self, start, string, cause)
+        fail(self, start, string, cause)
         raise
 
-    packrat_cache.set(lookup, tokens)
+    if self.parseAction and (doActions or self.parser_config.callDuringTry):
+        try:
+            for fn in self.parseAction:
+                tokens = fn(tokens, start, string)
+        except Exception as cause:
+            fail(self, start, string, cause)
+            raise
+    match(self, loc, tokens.end, string, tokens)
+
     return tokens
 
 
