@@ -12,7 +12,7 @@ from __future__ import absolute_import, division, unicode_literals
 from mo_parsing.engine import Engine
 from moz_sql_parser.keywords import *
 from moz_sql_parser.utils import *
-from moz_sql_parser.row_clause import row_clause
+from moz_sql_parser.windows import window, sortColumn
 
 engine = Engine().use()
 engine.add_ignore(Literal("--") + restOfLine)
@@ -25,14 +25,17 @@ sqlserver_ident = Regex(r"\[(\]\]|[^\]])*\]").addParseAction(unquote)
 ident = Combine(
     ~RESERVED
     + (delimitedList(
-        Literal("*") | literal_string | mysql_ident | sqlserver_ident | Word(IDENT_CHAR),
+        Literal("*")
+        | literal_string
+        | mysql_ident
+        | sqlserver_ident
+        | Word(IDENT_CHAR),
         separator=".",
         combine=True,
     ))
 ).set_parser_name("identifier")
 
 # EXPRESSIONS
-expr = Forward()
 
 # CASE
 case = (
@@ -60,7 +63,6 @@ cast = Group(
     CAST("op") + LB + expr("params") + AS + known_types("params") + RB
 ).addParseAction(to_json_call)
 
-
 _standard_time_intervals = MatchFirst([
     Keyword(d, caseless=True).addParseAction(lambda t: durations[t.lower()])
     for d in durations.keys()
@@ -72,7 +74,6 @@ interval = (
     INTERVAL + ("'" + delimitedList(duration) + "'" | duration)
 ).addParseAction(to_interval_call)
 
-
 timestamp = (
     time_functions("op")
     + (
@@ -84,7 +85,12 @@ timestamp = (
 ).addParseAction(to_json_call)
 
 extract = (
-    Keyword("extract", caseless=True)("op") + LB + (_standard_time_intervals | expr("params")) + FROM + expr("params") + RB
+    Keyword("extract", caseless=True)("op")
+    + LB
+    + (_standard_time_intervals | expr("params"))
+    + FROM
+    + expr("params")
+    + RB
 ).addParseAction(to_json_call)
 
 namedColumn = Group(
@@ -98,7 +104,11 @@ distinct = (
 ordered_sql = Forward()
 
 call_function = (
-    ident("op") + LB + Optional(Group(ordered_sql) | delimitedList(expr))("params") + RB
+    ident("op")
+    + LB
+    + Optional(Group(ordered_sql) | delimitedList(expr))("params")
+    + Optional(Keyword("ignore", caseless=True) + Keyword("nulls", caseless=True))("ignore_nulls")
+    + RB
 ).addParseAction(to_json_call)
 
 compound = (
@@ -106,7 +116,6 @@ compound = (
     | TRUE
     | FALSE
     | NOCASE
-    | (DATE("op") + sqlString("params")).addParseAction(to_json_call)
     | interval
     | timestamp
     | extract
@@ -116,11 +125,11 @@ compound = (
     | distinct
     | (LB + Group(ordered_sql) + RB)
     | (LB + Group(delimitedList(expr)).addParseAction(to_tuple_call) + RB)
-    | realNum.set_parser_name("float")
-    | intNum.set_parser_name("int")
     | sqlString.set_parser_name("string")
     | call_function
     | known_types
+    | realNum.set_parser_name("float")
+    | intNum.set_parser_name("int")
     | ident
 )
 
@@ -145,27 +154,6 @@ alias = (
     .addParseAction(to_alias)
 )
 
-# SQL STATEMENT
-sortColumn = expr("value").set_parser_name("sort1") + Optional(
-    DESC("sort") | ASC("sort")
-) | expr("value").set_parser_name("sort2")
-
-window = Optional(
-    WITHIN_GROUP
-    + LB
-    + Optional(ORDER_BY + delimitedList(Group(sortColumn))("orderby"))
-    + RB
-)("within") + Optional(
-    OVER
-    + LB
-    + Optional(PARTITION_BY + delimitedList(Group(expr))("partitionby"))
-    + Optional(
-        ORDER_BY
-        + delimitedList(Group(sortColumn))("orderby")
-        + Optional(row_clause)("range")
-    )
-    + RB
-)("over")
 
 selectColumn = (
     Group(
@@ -216,22 +204,19 @@ unordered_sql = (
 ordered_sql << (
     (
         Group(unordered_sql)
-        + (
-            OneOrMore(UNION_ALL + Group(unordered_sql))
-            | ZeroOrMore(Group(UNION) + Group(unordered_sql))
-        )
+        + ZeroOrMore((UNION_ALL | UNION) + Group(unordered_sql))
     )("union")
     + Optional(ORDER_BY + delimitedList(Group(sortColumn))("orderby"))
     + Optional(LIMIT + expr("limit"))
     + Optional(OFFSET + expr("offset"))
 ).set_parser_name("ordered sql").addParseAction(to_union_call)
 
-
-statement = (
+statement = Forward()
+statement << (
     Optional(
         WITH
         + delimitedList(Group(
-            ident("name") + AS + LB + Group(ordered_sql)("value") + RB
+            ident("name") + AS + LB + statement("value") + RB
         ))
     )("with")
     + Group(ordered_sql)("query")
